@@ -85,7 +85,4 @@ pub enum CompilationErrorPayload {
 
     #[error("Too many `super.` calls.")]
     SuperLimitReached,
-
-    #[error("Card by id {card_id} not found")]
-    MissingCard { card_id: super::CardId },
 }
